#ifndef CMDGVISION_H
/** \file

   Definitions to handle data in various GammaVision formats.

*/
/*

   Copyright  2005-2006 M.Kraemer, GSI, D-64291 Darmstadt

   $Log: cmdgvision.h,v $
   Revision 1.1.2.1  2005/11/22 17:16:48  kraemer
   sync'ed with old CMD


*/
#define CMDGVISION_H

#include "cmderr.h"

                                      /* global flags */
#define CMDGVISION_DEBUG  0x80000000    /* general check flag */
#define CMDGVISION_QUIET  0x10000000    /* suppress error messages */
#define CMDGVISION_NOMSG  CMDGVISION_QUIET

#define CMDGVISION_MAXDIM 2

struct SCMDGVISIONWDW {
   long lvLim[CMDGVISION_MAXDIM][4];  
};
                                      /* various input file formats */
enum { CMDGVISION_FMTNONE=0, CMDGVISION_FMTSPE, CMDGVISION_FMTTXT, };
                                      /* structure to store data from file */
struct SCMDGVISION {
   unsigned long ulFlag;              /**< hold general flag bits */
   unsigned long ulSpace;
   unsigned long ulFmt;               /**< input file format */
   char *pcId;                        /**< spec ID */
   char **pvcRem;                     /**< [nRem] spec remarks */ 
   int  nRem;
   char *pcFile;                      /**< file name */    
   char *pcDate;                      /**< acq start date */    
   char *pcTime;                      /**< acq start time */    
   char *pcUnit;                      /**< calibration units */

   int  nEff;
   double *pdEff;                     /**< [nEff] efficiency terms */     

   int  nMCA;
   double *pdMCA;                     /**< [nMCA] MCA calibration terms */     

   int  nShape;
   double *pdShape;                   /**< [nShape] shape calibration terms */     

   double dvEFit[3];                  /**< energy calibr. coeff. */
   double dvFFit[3];                  /**< FWHM calibr. coeff. ??? */
   double dvTime[2];                  /**< measuring time, [0/1] = real/live */ 
   int nWdw;
   struct SCMDGVISIONWDW *psWdw;      /**< [nWdw] window limits */ 
   long   lvLim[CMDGVISION_MAXDIM][4];/**< spectrum limits */
   long lSum;                         /**< spectrum sum (on read) */ 
   long *plSpc;                       /**< spectrum data */ 
   struct SCMDERR sCE;
};

/*

   Functions
   ---------
   
   cmdGVisionFree    ( gvision, flag )
                     o releases resources
                   3 o to do
                     o calls:
                       o cmdGVisionInit()
                            
   cmdGVisionInit    ( gvision, flag )
                     o initialize
                   2 o to do
                     o calls:
                         
   cmdGVisionInspect ( gvision, flag )
                     o inspect 
                   1 o to do
                     o calls:
                         
   cmdGVisionRead    ( gvision, filename, flag )
                     o reads gammavision file
                   4 o to do
                     o calls:
                       o cmdGVisionInit     
                       o cmdGVisionFree     

   cmdGVisionWrite   ( gvision, filename, flag )
                     o writes gammavision file
                   5 o to do
                     o calls:
*/
                                      /* function prototypes */
int cmdGVisionFree   ( struct SCMDGVISION *, unsigned long );
int cmdGVisionInit   ( struct SCMDGVISION *, unsigned long );
int cmdGVisionInspect( struct SCMDGVISION *, unsigned long );
int cmdGVisionRead   ( struct SCMDGVISION *, const char *, unsigned long );

struct SCMDGVISION *_cmdGVisionTest( unsigned long );

                                      /* error handling */
int cmdGVisionError( struct SCMDERR *, unsigned long );

enum { 
   ERRCMDGVISION_SYS=1,
   ERRCMDGVISION_BADADDR,
   ERRCMDGVISION_BADSYNTAX,
   ERRCMDGVISION_UNKNOWN,
   ERRCMDGVISION_BADFLOAT,
   ERRCMDGVISION_BADINTEGER,
   ERRCMDGVISION_BADTIME,
   ERRCMDGVISION_BADCHANNEL,
   ERRCMDGVISION_DATAEXTRANEOUS,
   ERRCMDGVISION_DATAMISSING,
   ERRCMDGVISION_BADNROI,
   ERRCMDGVISION_BADNCOEFF,
   ERRCMDGVISION_WDWMISSING,
};

#endif
