#ifndef TRXGEO_H
#define TRXGEO_H
/** \file
   Geometry definitions.
*/
/*

   Copyright  2000-2009 M.Kraemer, GSI Darmstadt 

   $Log: trxgeo.h,v $
   Revision 1.3  2008/06/20 14:29:36  kraemer
   tilted vol's

   Revision 1.2  2002/11/25 16:27:27  kraemer
   2file, density, cutoff, e/bfld

   Revision 1.1  2001/01/09 14:54:31  kraemer
   *** empty log message ***


*/

#include "m.h"
#include "trxvol.h"

                                      /*  */
#define MTRXGEOINC 16

                                      /** \struct STRXGEO
                                          \brief Describes geometry.
                                      */ 
struct STRXGEO {
   unsigned long ulFlag;              /**< flag bits */
   unsigned long ulSpace;             /**< space counter */
   int nV, mV;                        /**< current, max # of volumes in geometry */
   struct STRXVOL *psV;               /**< [mV] volumes */
   struct SCMDERR sCE;                /**< error handling */
};
                                      /* */
#define TRXGEO_REFCOPY 0x00000001     /* don't free() */

#define TRXGEO_FILEVERSION 20000118
#define TRXGEO_FILETYPE    "trxgeo"

/*

   Functions:
   ----------

   TRXGeoCopyByList   ( geoto, geofr, vol-list, flag )
                      ref copy only, no renumbering, no re-surfacing
                    6 x to test
                      o calls  
                        o TRXGeoInit()
                        o TRXGeoFree()
                        o TRXVolCopy()
 
   TRXGeoDelete       ( geo, vol-list, flag )
                   13 o to test
                      o delete volumes by name
                      o calls:
                        o TRXVolFree()
                        o TRXGeoFree()
                           
   TRXGeoExportgd     ( geo, prefix, vol-list, flag )
                   12 o to do
                      o export in gd format
                      o calls
                        o TRXGeoCopyByList()   
                        o TRXGeoFree()   
                        o TRXGeoInit()   
                        o TRXGeo2Filegd();
                        
   TRXGeoFindName     ( geo, name, flag )
                    5 x to test
                      o calls                           
                                                    
   TRXGeoFree         ( geo, flag )
                    3 x to test
                      o calls  
                        o TRXGeoInit()
                        o TRXVolFree()
                                                    
   TRXGeoInit         ( geo, flag )  
                    2 x to test
                      o calls  
                                                    
   TRXGeoInspect      ( geo, flag )
                    1 x to test
                      o calls  
                        o TRXVolInspect();
                        
   TRXGeoIsValid      ( geo, flag )
                    4 x to test
                      o calls  
                        o TRXVolIsValid();
                        
   TRXGeoList         ( geo, vol-list, flag )
                    8 x to test
                      o calls  
                        o TRXGeoCopyByList()   
                        o TRXGeoFree()   
                        o TRXGeoInit()   
                        
   TRXGeoRead         ( geo, file-list, flag )
                      o read geometry file(s)
                   10 x to test
                      o calls
                        o TRXGeoFree()
                        o TRXGeoInit()
                        o TRXGeoFindName()
                        o TRXVol*()
                           
   TRXGeoSpace        ( geo, flag )
                      o estimate space requirements
                   14 x to test
                      o calls  
                        o TRXVolSpace()   
                        
   TRXGeoWrite        ( geo, prefix, vol-list, flag )
                    9 x to test
                      o writes the current geometry setup
                      o calls  
                        o TRXGeoCopyByList()   
                        o TRXGeoFree()   
                        o TRXGeoInit()   
                        o TRXGeo2File();

   TRXGeo2File        ( geo, fp, flag )
                      o output geometry data, native format
                    7 x to test
                      o calls
                        o TRXGeoIsValid
                        o TRXVol*()  

   TRXGeo2Filegd      ( geo, fp, flag )
                      o output geometry data, gd format
                   11 o to do
                      o calls  

   Functions to handle score counters
   ----------------------------------
   
   TRXGeoScoreEvaluate   ( geo, vol-list, nevt, flag )
                       4 x to test
                         o resets counters
                         o calls
                           o TRXVolScoreEvaluate() 

   TRXGeoScoreFanoAlloc  ( geo, nevt, flag )
                       3 x to test
                         o allocates/prepares arrays for Fano factor
                         o calls:
                           o TRXVolScoreFanoAlloc() 
                         
   TRXGeoScoreFanoFree   ( geo, flag )
                       2 x to test
                         o frees Fano arrays
                           o TRXVolScoreFanoFree () 
                         
   TRXGeoScoreReset      ( geo, list, flag )
                       1 x to test 
                         o resets counters 
                           o TRXVolScoreReset() 


   TRXGeoError           error handling
         
*/
                                      /* Common status and action flags */
                                      /* masks */
#define TRXGEO_MASKGLOBAL 0xFF000000  /* global (e.g. debug) */
#define TRXGEO_MASKFUNC   0x00FF0000  /* function specific flags */
#define TRXGEO_MASKQUAL   0x0000FFFF  /* for additional qualifiers */

                                      /* flags */
#define TRXGEO_DEBUG      0x80000000  /* general check flag */
#define TRXGEO_CHECK      TRXGEO_DEBUG/* general check flag */
#define TRXGEO_QUIET      0x10000000  /* suppress error messages */
#define TRXGEO_NOMSG      TRXGEO_QUIET

                                      /* function prototypes */
int TRXGeoCopyByList( struct STRXGEO *, struct STRXGEO *, const char *, unsigned long );
int TRXGeoDelete    ( struct STRXGEO *, const char *, unsigned long );
int TRXGeoExportgd  ( struct STRXGEO *, const char *, const char *, unsigned long );
struct STRXVOL *TRXGeoFindName  ( struct STRXGEO *, const char *, unsigned long );
int TRXGeoFree      ( struct STRXGEO *, unsigned long );
int TRXGeoInit      ( struct STRXGEO *, unsigned long );
int TRXGeoInspect   ( struct STRXGEO *, unsigned long );
int TRXGeoIsValid   ( struct STRXGEO *, unsigned long );
int TRXGeoList      ( struct STRXGEO *, const char *, unsigned long );
int TRXGeoRead      ( struct STRXGEO *, const char *, unsigned long );
int TRXGeoSpace     ( struct STRXGEO *, unsigned long );
int TRXGeoWrite     ( struct STRXGEO *, const char *, const char *, unsigned long );

int TRXGeo2File     ( struct STRXGEO *, FILE *, unsigned long );
int TRXGeo2Filegd   ( struct STRXGEO *, FILE *, unsigned long );

int TRXGeoScoreEvaluate ( struct STRXGEO *, const char *, unsigned long, unsigned long );
int TRXGeoScoreFanoAlloc( struct STRXGEO *, unsigned long, unsigned long  );
int TRXGeoScoreFanoFree ( struct STRXGEO *, unsigned long );
int TRXGeoScoreReset    ( struct STRXGEO *, const char *, unsigned long );

struct STRXGEO *_TRXGeoTest( unsigned long );
struct STRXGEO *_TRXGeoTestBox ( unsigned long );
struct STRXGEO *_TRXGeoTestSphere( unsigned long );

                                      /* error handling */
int TRXGeoError( struct SCMDERR *, unsigned long );

enum { ERRTRXGEO_SYS=1,
       ERRTRXGEO_BADADDR,
       ERRTRXGEO_BADNVOL,
       ERRTRXGEO_BADVOL,
       ERRTRXGEO_BADITEMLIST,
       ERRTRXGEO_VOLCOPYFAILED, 
       ERRTRXGEO_NOVOLS,
       ERRTRXGEO_VOLMATCHFAILED, 
       ERRTRXGEO_BADFILETYPE,
       ERRTRXGEO_BADFILEVERSION,
       ERRTRXGEO_BADSYNTAX,
       ERRTRXGEO_BADNUMBER,
       ERRTRXGEO_BADSHAPE,
       ERRTRXGEO_VOLNAMENOTUNIQUE,
       ERRTRXGEO_SHAPENOTYET,
       ERRTRXGEO_BADHEIGHT,
       ERRTRXGEO_BADRADIUS,
       ERRTRXGEO_BADANGLE,
       ERRTRXGEO_BADSIZE,
       ERRTRXGEO_BADTHICKNESS,
       ERRTRXGEO_UNKNOWNTAG,
       ERRTRXGEO_BADNEVT,
       ERRTRXGEO_NOSUCHVOL,   
       ERRTRXGEO_BADDENSITY,
       ERRTRXGEO_BADCUTOFFPROJ,
};

#endif
